# All the necessary values for the algorithm will
# be initialized in this file

# remark that the Interpolationpoints are not placed
# in a matrix but in a vector of vectors
# instead of using [i,j] we use here [i][j]



#################################################################################
#										#
# Only the previous part may be changed						#
#										#
#################################################################################	
#										#
# The next part starts with all the declarations				#
#										#
#################################################################################



# Matrix dimensions, initialization
nB:=numvars+2;


# construction of sdeg (we will store this one in ssdeg

ssdeg:=deg[numvars+2]*[seq(1,i=1..numvars+2)]-deg;

# number of interpolationpoints
numip:=0;
for i from 1 to (numvars+1) do
  numip:=numip+binomial(numvars+ssdeg[i],ssdeg[i]);
od;

# Maximummatrix dimensions
cB:=numvars+(numvars-1)*numip+2:;
cBmax:=cB;

if ChoiceIP=1 then 
  # random in [-10,10]
  num:=numip+extra;
  IP:= vector(num,[seq(evalm(0.1*randvector(3)),i=1..num)]);
elif ChoiceIP=2 then 
  # random in [-1,1]
  num:=numip+extra;
  IP:= vector(num,[seq(evalm(0.01*randvector(3)),i=1..num)]);
elif ChoiceIP=3 then
  # Using Chebyshev interpolation points in;
  num:=ceil(evalf((numip+extra)^(1/numvars)));
  ipvector:=[seq(evalf(cos(i*Pi/(num-1))),i=0..(num-1))];  
  num:=num^numvars;
  IP:=vector(num,[seq(0,i=1..num)]);;
  T:=cartprod([seq((ipvector),i=1..numvars)]);
  i:=1;
  while not T[finished] do
    IP[i]:= T[nextvalue](); 
    i:=i+1;
  end do;
  unassign('i');
  unassign('T');
  unassign('ipvector');

elif ChoiceIP=4 then
  # Using equal distributed  interpolation points in [1,1];
  num:=ceil(evalf((numip+extra)^(1/numvars)));
  ipvector:=[seq(evalf(-1+i*(2/(num-1))),i=0..(num-1))];  
  num:=num^numvars;
  IP:=vector(num,[seq(0,i=1..num)]);
  T:=cartprod([seq((ipvector),i=1..numvars)]);
  i:=1;
  while not T[finished] do
    IP[i]:= T[nextvalue](); 
    i:=i+1;
  end do;
  unassign('i');
  unassign('T');
  unassign('ipvector');
fi;


# we add one more interpolationpoint cause the algorithm tests the residuals, 
# in a new interpolation point,
# and when numip iterations are performed (which is the maximum) 
# then the residual wil be calculated useng the next iteration point: numip +1
 

if verbose > -2 then
###################### INFORMATION PRINT #########################

if verbose >= 0 then
  print("################## PROGRAM CHARACTERISTICS ############################");
  print("number of Interpolationpoints needed:",numip);
fi;
fi;
numip:=num;
if verbose >-2 then
if verbose >=0 then
  print("number of Interpolationpoints generated:",num);
  print("Maximum number of columns for B:",cB );
  print("variables:",vars);
  print("degree of P:", [seq(deg[i],i=1..numvars+1)], "degree of V:", deg[numvars+2]);
  print("support",ssdeg);
  if ChoiceIP=1 then 
    print("Using Random Interpolation Points in [-10,10]"):;
  elif ChoiceIP=2 then 
    print("Using Random Interpolation Points in [-1,1]");
  elif ChoiceIP=3 then
    print("Using Chebyshev interpolation points in [-1,1]");
  elif ChoiceIP=4 then
    print("Using equal spaced interpolationpoints in [-1,1]");
  fi;
  print("for constructing the grid");
  print("Maple counts with:",Digits," Digits");
  print("Stop when residual is smaller then:",residual);
  print("Omitting interpolation point when smaller then:", omitres); 
fi;

if verbose>=2 then
  print("function to solve:");
  print(eval(seq(P[i],i=1..vectdim(P)-1)));
  print("with righthandside:");
  print(eval(P[vectdim(P)]));
fi;

if verbose >=0 then
 print("############ press a button to continu ########################################"); 
 #readline();
fi;
fi;


# Some values, which are not useful anymore
# will be killed:
unassign('cB');
unassign('num');