########################################################
#							#
#    SOLVE THE MULTIVARIATE INTERPOLATIONPROBLEM	#     
#							#
#########################################################



# Here we start in fact the real procedure
# for solving the multivariate interpolationproblem
# we assume several things
# the interpolationpoints are fixed
# the polynomial is loaded in the variable P




Solve:=proc(vars,numvars,nB,P,IP,numip,ssdeg,residual,omitres,verbose)
	local i,j,resB,pivot,IPpoint,mxx,cB,B,stoploop,ct,sdeg,time1,time2,
        timeupdate,Q,mxxx;


	###################### INITIALIZATION #############################
	# some variables get initialized
	ct:=1;
	cB:=nB;
	B := matrix([seq([seq(Krok([i,j]),j=1..nB)],i=1..nB)]);
	resB:=constructresB(vars,numvars,IP,cB,P);
	sdeg:=ssdeg;
	sdeg:=matadd(sdeg,vector(cB,[seq(1,i=1..cB-1),0])):;
	stoploop:=0; 
	timeupdate:=0;

	################## END OF INITIALIZATION#############################


        if verbose>=1 then
        print("#################### SOLVING MULTIVARIATE INTERPOLATION #####################"):;
	fi;


	while not(iszero(sdeg)) and (stoploop=0) do
 	  
	  pivot,IPpoint,mxx:=searchpivot(cB,numip,sdeg,resB):;
	  Q:=[seq(coeffs(B[i,pivot]),i=1..rowdim(B))];
          mxxx:=max(seq(abs(Q[i]),i=1..vectdim(Q)));
	  unassign('Q');
          
          # mxx:=mxx/max(seq(abs(resB[IPpoint,i]),i=[seq(j,j=1..pivot-1),seq(j,j=pivot+1..cB)]));
          # mxx:=abs(mxx);	  
          
          if verbose>=1 then
	  print("Step:", ct,"Point:",IPpoint,"pivot:",pivot,"Factor",mxx/mxxx,"Size B",cB,"max sdegree",max(seq(sdeg[i],i=1..cB)));
            if verbose > 1 then
              print("size pivot", mxx, " size of max coefficient",mxxx, "fraction between the two ",mxx/mxxx); 
             fi;
          fi;
 
	  if  (mxx/mxxx)<omitres then
	    if verbose >=0 then
	      print("Factor",mxx/mxx,"smaller then", omitres,"step",ct, "not executed");
              print("column", pivot, "will be omitted this step");
              #print("with maximum residual",max(seq(abs(resB[i,pivot]),i=1..numip)));
            fi;
	     #for i from 1 to numip do
              # resB[i,pivot]:=0;
	     #od;
            sdeg[pivot]:=sdeg[pivot]-1;
          else
	    time1:=time();
            B,cB,resB:=IteratInterpol(vars,numvars,nB,cB,B,IP,IPpoint,pivot,resB,numip);
 	    sdeg[pivot]:=sdeg[pivot]-1;
            sdeg:=vector(cB,[seq(sdeg[i],i=1..cB-numvars+1),seq(sdeg[pivot],i=1..numvars-1)]);
            B,cB,resB,sdeg:=cleanupB(B,resB,sdeg,cB,omitres,residual,numip);
            time2:=time();
            timeupdate:=timeupdate+time2-time1;
  	    ct:=ct+1;
	  fi;

	  if max(seq(abs(resB[i,nB]),i=1..numip))<residual then
	    stoploop:=1;
            if verbose>= 0 then
	    print("Residual",max(seq(abs(resB[i,nB]),i=1..numip)),"smaller then",residual,"stopping execution");
            fi;
	  end;


          
	od;
        if verbose >= 0 then
	print("######################################################################"):;

	print("End of procedure with maximum Residual:",max(seq(resB[i,nB],i=1..numip)));
	print("After ",ct-1,"steps and B a", nB, "x",cB, "matrix");
	print("######################################################################");
        fi;
	RETURN(col(B,nB),B,cB,resB,timeupdate,ct);
end:













##############################################################################################



#########################################################		
#							#
#							#
# THIS PROCEDURE SEARCHES THE PIVOT			#
#							#	
#							#
########################################################		

searchpivot:=proc(cB,numip,sdeg,resB)
	local i,j,mx,mxx,pivot,IPpoint; 


	# initialization of some vars
    	mx:=max(seq(sdeg[j],j=1..cB));
	mxx:=0;
	

	for i from 1 to cB do
	  if sdeg[i]=mx then
	    for j from 1 to numip do
	      if mxx<abs(resB[j,i]) then
		mxx:=abs(resB[j,i]);
		pivot:=i;
		IPpoint:=j;
	      fi;
	    od;
	  fi;
	od;
	 
	RETURN(pivot,IPpoint,mxx);
end;


########################################################################################
